package com.nationrel.modules.rga.enums;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.nationrel.modules.rga.constant.RgaConstants;
import com.nationrel.modules.rga.general.domain.enums.reliaqube.ReliaqubeLanguageKeyEnum;
import com.nationrel.modules.rga.general.util.common.BasicDataUtil;

import java.util.HashMap;
import java.util.Map;

/**
 * 分布类型
 * @author Aimee on 2020/10/22.
 */
public enum RgaDistributingCodeEnum {
    /**
     * 分布类型
     */
    Weibull("0", "威布尔", "Weibull", "Weibull"),
    Lognormal("8", "对数正态", "LogNormal", "Lognormal");

    private String index;

    private String name;

    private String englishName;

    private String windowsEnglishName;

    private RgaDistributingCodeEnum(String index, String name, String englishName, String windowsEnglishName) {
        this.index = index;
        this.name = name;
        this.englishName = englishName;
        this.windowsEnglishName = windowsEnglishName;
    }

    public String getIndex() {
        return index;
    }

    public void setIndex(String index) {
        this.index = index;
    }

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public String getEnglishName() {
        return englishName;
    }

    public void setEnglishName(String englishName) {
        this.englishName = englishName;
    }

    public String getWindowsEnglishName() {
        return windowsEnglishName;
    }

    public void setWindowsEnglishName(String windowsEnglishName) {
        this.windowsEnglishName = windowsEnglishName;
    }

    // Implementing a fromString method on an enum type
    private static final Map<String, RgaDistributingCodeEnum> JSON_ARRAY_MAP = new HashMap<String, RgaDistributingCodeEnum>();

    public static final JSONArray JSON_ARRAY = new JSONArray();

    static {
        // Initialize map from constant name to enum constant
        for (RgaDistributingCodeEnum enumForEach : values()) {
            JSON_ARRAY_MAP.put(enumForEach.getIndex(), enumForEach);
            JSONObject jsonObject = new JSONObject();
            jsonObject.put("value", enumForEach.toString());
            jsonObject.put("key", enumForEach.getIndex());
            JSON_ARRAY.add(jsonObject);
        }
    }

    public static String getNameByLanguageKey(String index, Integer languageKey) {
        for (RgaDistributingCodeEnum e : values()) {
            if (e.getIndex().equals(index)) {
                return languageKey.equals(ReliaqubeLanguageKeyEnum.中文.getIndex()) ? e.getName() : e.getEnglishName();
            }
        }
        return BasicDataUtil.EMPTY_STR;
    }

    public static String getEnglishNameByIndex(String serverSystem, String index) {
        for (RgaDistributingCodeEnum e : values()) {
            if (e.getIndex().equals(index)) {
                return RgaConstants.SERVER_SYSTEM_LINUX.equals(serverSystem) ? e.getEnglishName() : e.getWindowsEnglishName();
            }
        }
        return BasicDataUtil.EMPTY_STR;
    }

    public static String getIndexByEnglishName(String englishName) {
        for (RgaDistributingCodeEnum e : values()) {
            if (e.getEnglishName().equals(englishName)) {
                return e.getIndex();
            }
        }
        return BasicDataUtil.EMPTY_STR;
    }

}
