package com.nationrel.modules.rga.enums;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.nationrel.modules.rga.general.domain.enums.reliaqube.ReliaqubeLanguageKeyEnum;
import com.nationrel.modules.rga.general.util.common.BasicDataUtil;

import java.util.HashMap;
import java.util.Map;

/**
 * 应力模型
 * @author Aimee on 2020/10/22.
 */
public enum RgaLifeStressModelEnum {
    /**
     * 应力模型
     */
    Arrhenius("0", "Arrhenius", "Arrhenius", "Arrhenius"),
    Eyring("1", "Eyring", "Eyring", "Eyring"),
    InversePowerLaw("2", "逆幂率", "Inverse power law", "IPL"),
    TemAndSid("3", "温度--湿度", "Temperature--humidity", "Temp_Humidity"),
    TemAndNotHot("4", "温度--非热", "Temperature--non thermal", "Temp_NonThermal"),
    GeneralLogLinear("5", "General_Log_Linear", "General_Log_Linear", "General_Log_Linear");

    private String index;

    private String name;

    private String englishName;

    private String apiRespName;

    private RgaLifeStressModelEnum(String index, String name, String englishName, String apiRespName) {
        this.index = index;
        this.name = name;
        this.englishName = englishName;
        this.apiRespName = apiRespName;
    }

    public String getIndex() {
        return index;
    }

    public void setIndex(String index) {
        this.index = index;
    }

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public String getEnglishName() {
        return englishName;
    }

    public void setEnglishName(String englishName) {
        this.englishName = englishName;
    }

    public String getApiRespName() {
        return apiRespName;
    }

    public void setApiRespName(String apiRespName) {
        this.apiRespName = apiRespName;
    }

    // Implementing a fromString method on an enum type
    private static final Map<String, RgaLifeStressModelEnum> JSON_ARRAY_MAP = new HashMap<String, RgaLifeStressModelEnum>();

    public static final JSONArray JSON_ARRAY = new JSONArray();

    static {
        // Initialize map from constant name to enum constant
        for (RgaLifeStressModelEnum enumForEach : values()) {
            JSON_ARRAY_MAP.put(enumForEach.getIndex(), enumForEach);
            JSONObject jsonObject = new JSONObject();
            jsonObject.put("value", enumForEach.toString());
            jsonObject.put("key", enumForEach.getIndex());
            JSON_ARRAY.add(jsonObject);
        }
    }

    public static String getNameByLanguageKey(String index, Integer languageKey) {
        for (RgaLifeStressModelEnum e : values()) {
            if (e.getIndex().equals(index)) {
                return languageKey.equals(ReliaqubeLanguageKeyEnum.中文.getIndex()) ? e.getName() : e.getEnglishName();
            }
        }
        return BasicDataUtil.EMPTY_STR;
    }

    public static String getIndexByApiName(String name) {
        for (RgaLifeStressModelEnum e : values()) {
            if (e.getApiRespName().equals(name)) {
                return e.getIndex();
            }
        }
        return BasicDataUtil.EMPTY_STR;
    }

    public static String getApiNameByIndex(String index) {
        for (RgaLifeStressModelEnum e : values()) {
            if (e.getIndex().equals(index)) {
                return e.getApiRespName();
            }
        }
        return BasicDataUtil.EMPTY_STR;
    }

}
