package com.nationrel.modules.rga.general.exception;

import com.nationrel.modules.rga.general.common.ResponseStatus;
import com.nationrel.modules.rga.general.domain.enums.alta.AltaCustomPromptEnum;
import com.nationrel.modules.rga.general.domain.enums.reliaqube.ReliaqubeParamDataCheckEnum;
import com.nationrel.modules.rga.general.domain.enums.reliaqube.ReliaqubeProgramExceptionEnum;
import com.nationrel.modules.rga.general.domain.enums.weibull.EwbCommonUsedEnum;
import com.nationrel.modules.rga.general.domain.enums.weibull.EwbCustomPromptEnum;

import java.math.BigDecimal;

/**
 * @author Hcs on 2018/8/6.
 */
public class DescribeException extends RuntimeException {

    private Integer status;

    /**
     * 继承exception，加入错误状态值
     *
     * @param status
     */
    public DescribeException(ResponseStatus status) {
        super(status.getMessage());
        this.status = status.getStatus();
    }

    /**
     * 自定义错误信息
     *
     * @param message
     * @param status
     */
    public DescribeException(String message, Integer status) {
        super(message);
        this.status = status;
    }

    /**
     * 数据校验异常错误信息
     *
     * @param enumStr
     * @param languageKey
     */
    public DescribeException(ReliaqubeParamDataCheckEnum enumStr, Integer languageKey) {
        super(ReliaqubeParamDataCheckEnum.getLanguageMessageByKey(enumStr, languageKey));
        this.status = ReliaqubeProgramExceptionEnum.ParamterDataException.getIndex();
    }

    /**
     * 数据校验异常错误信息
     *
     * @param enumStr
     * @param languageKey
     */
    public DescribeException(EwbCustomPromptEnum enumStr, Integer languageKey) {
        super(EwbCustomPromptEnum.getLanguageMessageByKey(enumStr, languageKey));
        this.status = ReliaqubeProgramExceptionEnum.ParamterDataException.getIndex();
    }

    /**
     * Alta数据校验异常错误信息
     *
     * @param enumStr
     * @param languageKey
     */
    public DescribeException(AltaCustomPromptEnum enumStr, Integer languageKey, Integer status) {
        super(AltaCustomPromptEnum.getLanguageMessageByKey(enumStr, languageKey));
        this.status = status;
    }

    /**
     * Alta数据校验异常错误信息
     *
     * @param enumStr
     * @param languageKey
     */
    public DescribeException(ReliaqubeParamDataCheckEnum enumStr, Integer languageKey, Integer status) {
        super(ReliaqubeParamDataCheckEnum.getLanguageMessageByKey(enumStr, languageKey));
        this.status = status;
    }

    /**
     * 数据校验异常错误信息
     *
     * @param enumStr
     * @param languageKey
     */
    public DescribeException(EwbCustomPromptEnum enumStr, Integer languageKey, Integer status) {
        super(EwbCustomPromptEnum.getLanguageMessageByKey(enumStr, languageKey));
        this.status = status;
    }

    /**
     * 数据校验异常错误信息
     *
     * @param enumStr
     * @param languageKey
     */
    public DescribeException(EwbCommonUsedEnum enumStr, Integer languageKey, EwbCustomPromptEnum enumStr1) {
        super(EwbCommonUsedEnum.getLanguageMessageByKey(enumStr, languageKey) + EwbCustomPromptEnum.getLanguageMessageByKey(enumStr1, languageKey));
        this.status = ReliaqubeProgramExceptionEnum.ParamterDataException.getIndex();
    }

    /**
     * 数据校验异常错误信息
     *
     * @param enumStr
     * @param languageKey
     */
    public DescribeException(int number, EwbCustomPromptEnum enumStr, Integer languageKey) {
        super(ReliaqubeParamDataCheckEnum.getConstantByLanguageKey("第", languageKey, number) + EwbCustomPromptEnum.getLanguageMessageByKey(enumStr, languageKey));
        this.status = ReliaqubeProgramExceptionEnum.ParamterDataException.getIndex();
    }

    /**
     * 判断alta结束时间数据异常
     *
     * @param enumStr
     * @param languageKey
     */
    public DescribeException(int number, String endTimeName, EwbCustomPromptEnum enumStr, Integer languageKey) {
        super(ReliaqubeParamDataCheckEnum.getConstantByLanguageKey("第", languageKey, number) + endTimeName + EwbCustomPromptEnum.getLanguageMessageByKey(enumStr, languageKey));
        this.status = ReliaqubeProgramExceptionEnum.ParamterDataException.getIndex();
    }

    /**
     * 数据校验异常错误信息
     *
     * @param enumStr
     * @param languageKey
     */
    public DescribeException(int number, ReliaqubeParamDataCheckEnum enumStr, Integer languageKey) {
        super(ReliaqubeParamDataCheckEnum.getConstantByLanguageKey("第", languageKey, number) + ReliaqubeParamDataCheckEnum.getLanguageMessageByKey(enumStr, languageKey));
        this.status = ReliaqubeProgramExceptionEnum.ParamterDataException.getIndex();
    }

    /**
     * 数据校验异常错误信息
     *
     * @param enumStr1
     * @param languageKey
     */
    public DescribeException(EwbCommonUsedEnum enumStr1, EwbCustomPromptEnum enumStr2, Integer languageKey) {
        super(EwbCommonUsedEnum.getLanguageMessageByKey(enumStr1, languageKey) + " " + EwbCustomPromptEnum.getLanguageMessageByKey(enumStr2, languageKey));
        this.status = ReliaqubeProgramExceptionEnum.ParamterDataException.getIndex();
    }

    /**
     * 数据校验异常错误信息
     *
     * @param enumStr1
     * @param languageKey
     */
    public DescribeException(EwbCommonUsedEnum enumStr1, EwbCustomPromptEnum enumStr2, String range, Integer languageKey) {
        super(EwbCommonUsedEnum.getLanguageMessageByKey(enumStr1, languageKey) + " " + EwbCustomPromptEnum.getLanguageMessageByKey(enumStr2, languageKey) + range);
        this.status = ReliaqubeProgramExceptionEnum.ParamterDataException.getIndex();
    }

    /**
     * 数据校验异常错误信息
     *
     * @param enumStr
     * @param languageKey
     */
    public DescribeException(String name, EwbCustomPromptEnum enumStr, Integer languageKey) {
        super(name + " " + EwbCustomPromptEnum.getLanguageMessageByKey(enumStr, languageKey));
        this.status = ReliaqubeProgramExceptionEnum.ParamterDataException.getIndex();
    }

    /**
     * 数据校验异常错误信息
     *
     * @param enumStr
     * @param languageKey
     */
    public DescribeException(String name, BigDecimal min, BigDecimal max, EwbCustomPromptEnum enumStr, Integer languageKey) {
        super(name + " " + EwbCustomPromptEnum.getLanguageMessageByKey(enumStr, languageKey) + ":" + min + "-" + max);
        this.status = ReliaqubeProgramExceptionEnum.ParamterDataException.getIndex();
    }

    /**
     * 数据校验异常错误信息
     *
     * @param enumStr
     * @param languageKey
     */
    public DescribeException(EwbCommonUsedEnum enumStr1, BigDecimal min, BigDecimal max, EwbCustomPromptEnum enumStr, Integer languageKey) {
        super(EwbCommonUsedEnum.getLanguageMessageByKey(enumStr1, languageKey) + " " + EwbCustomPromptEnum.getLanguageMessageByKey(enumStr, languageKey) + ":" + min + "-" + max);
        this.status = ReliaqubeProgramExceptionEnum.ParamterDataException.getIndex();
    }

    public Integer getStatus() {
        return status;
    }

    public void setStatus(Integer status) {
        this.status = status;
    }
}
