package com.nationrel.modules.rga.general.util.baidu;

import com.alibaba.fastjson.JSONObject;
import com.nationrel.modules.rga.general.domain.enums.reliaqube.ReliaqubeLanguageKeyEnum;
import com.nationrel.modules.rga.general.util.common.BasicDataUtil;
import com.nationrel.modules.rga.general.util.common.ObjectActionUtil;

import java.util.HashMap;
import java.util.Map;

public class BaiduTransApi {
    private static final String TRANS_API_HOST = "http://api.fanyi.baidu.com/api/trans/vip/translate";

    private String appid;
    private String securityKey;

    public BaiduTransApi(String appid, String securityKey) {
        this.appid = appid;
        this.securityKey = securityKey;
    }

    public String getTransResult(String query, String from, String to) {
        Map<String, String> params = buildParams(query, from, to);
        return HttpGet.get(TRANS_API_HOST, params);
    }

    private Map<String, String> buildParams(String query, String from, String to) {
        Map<String, String> params = new HashMap<String, String>();
        params.put("q", query);
        params.put("from", from);
        params.put("to", to);

        params.put("appid", appid);

        // 随机数
        String salt = String.valueOf(System.currentTimeMillis());
        params.put("salt", salt);

        // 签名加密前的原文
        String src = appid + query + salt + securityKey;
        params.put("sign", MD5.md5(src));

        return params;
    }

    public static String translation(String message, String from, String to){
        BaiduTransApi api = new BaiduTransApi("20200714000519111", "Y2cMLrgxec4jPvu69HOp");
        String apiresult = api.getTransResult(message, from, to);
        Object transResult = getJsonValueByKey(apiresult,"trans_result");
        if (ObjectActionUtil.isNotEmpty(transResult)){
            return getJsonValueByKey(transResult.toString().replace("[","").replace("]",""),"dst").toString();
        }
        return BasicDataUtil.EMPTY_STR;
    }

    public static String translation(String message, Integer from, Integer to){
        BaiduTransApi api = new BaiduTransApi("20200714000519111", "Y2cMLrgxec4jPvu69HOp");
        String apiresult = api.getTransResult(message, ReliaqubeLanguageKeyEnum.getLanguageCodeByIndex(from), ReliaqubeLanguageKeyEnum.getLanguageCodeByIndex(to));
        Object transResult = getJsonValueByKey(apiresult,"trans_result");
        if (ObjectActionUtil.isNotEmpty(transResult)){
            return getJsonValueByKey(transResult.toString().replace("[","").replace("]",""),"dst").toString();
        }
        return BasicDataUtil.EMPTY_STR;
    }

    /**
     * @Title getJsonValueByKey
     * @Description 获取Json格式字符串中key对应的值
     * @param jsonStr json格式的字符串
     * @param key 要获取值的键
     * @return Object
     * @version V1.0
     */
    public static Object getJsonValueByKey(String jsonStr, String key) {
        // 此处引入的是 com.alibaba.fastjson.JSONObject; 对象
        JSONObject jsonObject = JSONObject.parseObject(jsonStr);
        // 获取到 key 对应的值
        return jsonObject.get(key);
    }

}
