package com.nationrel.modules.rga.general.util.common;

import com.alibaba.fastjson.JSONObject;
import lombok.Data;
import org.apache.commons.lang3.StringUtils;
import org.springframework.util.CollectionUtils;

import java.io.IOException;
import java.lang.reflect.Array;
import java.math.BigDecimal;
import java.util.List;
import java.util.UUID;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import static java.util.regex.Pattern.compile;

/**
 * @author HuangKang on 2018/11/29.
 */
@Data
public class BasicDataUtil<T> {

    public static final String EMPTY_STR = "";

    /**
     * list转string并去除大括号
     *
     * @param str
     * @return
     */
    public static String parseEmptyStr(String str) {
        return StringUtils.isBlank(str) ? null : str;
    }

    /**
     * list转string并去除大括号
     *
     * @param str
     * @return
     */
    public static JSONObject parseJsonStr(String str) {
        return StringUtils.isBlank(str) ? null : JSONObject.parseObject(str);
    }

    /**
     * list转string并去除大括号
     *
     * @param list
     * @return
     */
    public static String listToString(List list) {
        return list.toString().replace("[", "").replace("]", "").replaceAll(" ", "");
    }

    /**
     * 生成32位UUID主键
     */
    public static String uuidGenerator() {
        return UUID.randomUUID().toString().replaceAll("-", "");
    }


    /**
     * 生成32位UUID主键
     */
    public static String parseInParam(String param) {
        return "'" + param + "'";
    }

    /**
     * 将带有方括号的字符串转为in查询条件
     *
     * @param param
     * @return
     */
    public static String parseInParam(List<String> param) {
        if (CollectionUtils.isEmpty(param)) {
            return null;
        }
        String paramStr = param.toString();
        paramStr = paramStr.replace("[", "").replace("]", "").replaceAll(" ", "");
        return "'" + paramStr.replaceAll(",", "','") + "'";
    }

    /**
     * 将带有方括号的字符串转为in查询条件
     *
     * @param param
     * @return
     */
    public static String parseInParamInt(List<Integer> param) {
        if (CollectionUtils.isEmpty(param)) {
            return null;
        }
        String paramStr = param.toString();
        return paramStr.replace("[", "").replace("]", "").replaceAll(" ", "");
    }

    /**
     * 匹配是否为数字
     *
     * @param str 可能为中文，也可能是-19162431.1254，不使用BigDecimal的话，变成-1.91624311254E7
     * @return
     */
    public static boolean isNumeric(String str) {
        // 该正则表达式可以匹配所有的数字 包括负数
        Pattern pattern = compile("-?[0-9]+(\\.[0-9]+)?");
        String bigStr;
        try {
            bigStr = new BigDecimal(str).toString();
        } catch (Exception e) {
            //异常 说明包含非数字。
            return false;
        }

        // matcher是全匹配
        Matcher isNum = pattern.matcher(bigStr);
        if (!isNum.matches()) {
            return false;
        }
        return true;
    }

    /**
     * object转double数组
     *
     * @param obj
     * @return
     * @throws IOException
     */
    public static Double[] objectToDoubleArray(Object obj) {
        int length = Array.getLength(obj);
        Double[] byteArray = new Double[length];
        for (int i = 0; i < byteArray.length; i++) {
            byteArray[i] = (Double) Array.get(obj, i);
        }
        return byteArray;
    }
}
