package com.nationrel.modules.rga.general.util.common;

import org.apache.commons.lang3.StringUtils;
import org.joda.time.DateTime;
import org.joda.time.format.DateTimeFormat;
import org.joda.time.format.DateTimeFormatter;

import java.text.ParseException;
import java.text.ParsePosition;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;

/**
 * @author hcs
 * 创建时间: 2018/2/7 0007.
 */
public class DateTimeUtil {

    public static final String STANDARD_FORMAT = "yyyy-MM-dd HH:mm:ss";
    public static final String STANDARD_FORMAT_NOS = "yyyy-MM-dd HH:mm";
    public static final String STANDARD_FORMAT_NOH = "yyyy-MM-dd";
    public static final String STANDARD_FORMAT_NOD = "yyyy-MM";
    public static final String STANDARD_FORMAT_HMS = "HH:mm:ss";

    /**
     * 定义私有构造函数来屏蔽这个隐式公有构造函数
     */
    private DateTimeUtil() {
    }

    public static String datesToStr(Date date1, Date date2) {
        if (date1 == null || date2 == null) {
            return StringUtils.EMPTY;
        }
        DateTime dateTime1 = new DateTime(date1);
        DateTime dateTime2 = new DateTime(date2);
        return dateTime1.toString("yyyy/MM/dd") + "-" + dateTime2.toString("yyyy/MM/dd");
    }

    public static Date strToDate(String dateTimeStr, String formatStr) {
        DateTimeFormatter dateTimeFormatter = DateTimeFormat.forPattern(formatStr);
        DateTime dateTime = dateTimeFormatter.parseDateTime(dateTimeStr);
        return dateTime.toDate();
    }

    /**
     * 获取后几天的时间
     *
     * @param date
     * @return
     */
    public static Date getBehindDay(Date date, int count) {
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(date);
        calendar.add(Calendar.DATE, count);
        return calendar.getTime();
    }

    /**
     * 获取后几天的时间
     *
     * @param date
     * @return
     */
    public static String getBehindDay(String date, int count, String pattern) {
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(strToDate(date, "yyyy-MM-dd"));
        calendar.add(Calendar.DATE, count);
        return dateToStr(calendar.getTime(), pattern);
    }

    /**
     * 获取传入日期的月份
     *
     * @param date
     * @return
     */
    public static int getMounth(Date date) {
        Calendar cal = Calendar.getInstance();
        cal.setTime(new Date());
        return cal.get(Calendar.MONTH) + 1;
    }

    /**
     * 将时间转为指定格式的字符串
     *
     * @param date
     * @param formatStr
     * @return
     */
    public static String dateToStr(Date date, String formatStr) {
        if (date == null) {
            return StringUtils.EMPTY;
        }
        DateTime dateTime = new DateTime(date);
        return dateTime.toString(formatStr);
    }

    /**
     * 将指定格式的日期字符串转换为日期对象。
     *
     * @param source  日期字符串。
     * @param pattern 模式。
     * @return Date 日期对象。
     */
    public static Date parseDate(String source, String pattern) {
        SimpleDateFormat sdf = new SimpleDateFormat(pattern);

        return sdf.parse(source, new ParsePosition(0));
    }

    /**
     * 将标准时间格式的字符串转为日期
     *
     * @param dateTimeStr
     * @return
     */
    public static Date strToDate(String dateTimeStr) {
        DateTimeFormatter dateTimeFormatter = DateTimeFormat.forPattern(STANDARD_FORMAT);
        DateTime dateTime = dateTimeFormatter.parseDateTime(dateTimeStr);
        return dateTime.toDate();
    }

    /**
     * 将时间转为标准格式的字符串
     *
     * @param date
     * @return
     */
    public static String dateToStr(Date date) {
        if (date == null) {
            return StringUtils.EMPTY;
        }
        DateTime dateTime = new DateTime(date);
        return dateTime.toString(STANDARD_FORMAT);
    }

    /**
     * 将当前时间转为yyyyMMdd格式的字符串
     *
     * @return
     */
    public static String dateToStr() {
        SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMdd");
        return sdf.format(new Date());
    }

    /**
     * 获取前几天的时间
     *
     * @param date
     * @return
     */
    public static Date getBeforeDay(Date date, int count) {
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(date);
        calendar.add(Calendar.DATE, -count);
        return calendar.getTime();
    }

    /**
     * 通过时间秒毫秒数判断两个时间的间隔
     *
     * @param date1
     * @param date2
     * @return
     */
    public static int getDayOfTwoDate(Date date1, Date date2) {
        date1 = getStartTime(date1);
        date2 = getStartTime(date2);
        int days = (int) ((date2.getTime() - date1.getTime()) / (1000 * 3600 * 24));
        return days;
    }

    /**
     * 获取下一天的时间
     *
     * @param date
     * @return
     */
    public static Date getNext2(Date date) {
        if (date == null) {
            return null;
        }
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(date);
        calendar.add(Calendar.DATE, 1);
        return calendar.getTime();
    }

    /**
     * 获取当前日期的开始时间
     *
     * @return
     */
    public static Date getStartTime(Date date) {
        Calendar todayStart = Calendar.getInstance();
        todayStart.setTime(date);
        todayStart.set(Calendar.HOUR_OF_DAY, 0);
        todayStart.set(Calendar.MINUTE, 0);
        todayStart.set(Calendar.SECOND, 0);
        todayStart.set(Calendar.MILLISECOND, 0);
        return todayStart.getTime();
    }

    /**
     * @param dtOld
     * @param add
     * @return Date
     * @Title: addDay
     * @Description: 在指定日期上加天数
     */
    public static Date addSomeDay(Date dtOld, int add) {
        if (dtOld == null) {
            return null;
        }
        Calendar cdate = Calendar.getInstance();
        cdate.setTime(dtOld);
        cdate.add(Calendar.DATE, add);
        return cdate.getTime();
    }

    /**
     * 获取当前日期的结束时间
     *
     * @return
     */
    public static Date getEndTime(Date date) {
        Calendar todayEnd = Calendar.getInstance();
        todayEnd.setTime(date);
        todayEnd.set(Calendar.HOUR_OF_DAY, 23);
        todayEnd.set(Calendar.MINUTE, 59);
        todayEnd.set(Calendar.SECOND, 59);
        todayEnd.set(Calendar.MILLISECOND, 999);
        return todayEnd.getTime();
    }

    /**
     * 判断是否是同一天
     *
     * @param d1
     * @param d2
     * @return
     */
    public static boolean isSameDate(Date d1, Date d2) {
        if (null == d1 || null == d2) {
            return false;
        }

        if (Math.abs(d1.getTime() - d2.getTime()) > 3600000 * 24) {
            return false;
        }

        Calendar c1 = Calendar.getInstance();
        c1.setTime(d1);
        Calendar c2 = Calendar.getInstance();
        c1.setTime(d2);
        if (c1.get(Calendar.DAY_OF_MONTH) != c2.get(Calendar.DAY_OF_MONTH)) {
            return false;
        }

        return true;
    }

    /**
     * 得到几个月之后的时间
     *
     * @param date
     * @param month
     * @return
     */
    public static String getBehindMonth(String date, int month, String pattern, String pattern2) {
        String nowDate = null;
        SimpleDateFormat format = new SimpleDateFormat(pattern);
        try {
            Date parse = format.parse(date);
            Calendar calendar = Calendar.getInstance();
            calendar.setTime(parse);
            calendar.add(Calendar.MONTH, month);
            format = new SimpleDateFormat(pattern2);
            nowDate = format.format(calendar.getTime());
        } catch (ParseException e) {
            e.printStackTrace();
        }
        return nowDate;
    }
}
