package com.nationrel.modules.rga.general.util.common;

import org.springframework.util.Base64Utils;

import javax.crypto.Cipher;
import javax.crypto.KeyGenerator;
import javax.crypto.SecretKey;
import javax.crypto.spec.SecretKeySpec;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.net.SocketException;
import java.security.NoSuchAlgorithmException;
import java.security.SecureRandom;
import java.util.Date;
import java.util.Scanner;
import java.util.logging.Level;
import java.util.logging.Logger;

public class EncryptionUtil {

    private static final String KEY_ALGORITHM = "AES";
    private static final String DEFAULT_CIPHER_ALGORITHM = "AES/ECB/PKCS5Padding";//默认的加密算法

    /**
     * AES 加密操作
     *
     * @param content 待加密内容
     * @param key     加密密钥
     * @return 返回Base64转码后的加密数据
     */
    public static String aesEncode(String content, String key) {
        try {
            // 创建密码器
            Cipher cipher = Cipher.getInstance(DEFAULT_CIPHER_ALGORITHM);
            byte[] byteContent = content.getBytes("utf-8");
            // 初始化为加密模式的密码器
            cipher.init(Cipher.ENCRYPT_MODE, getSecretKey(key));
            // 进行加密操作
            byte[] result = cipher.doFinal(byteContent);
            //通过Base64转码返回
            return new String(Base64Utils.encode(result));
        } catch (Exception ex) {
            Logger.getLogger(EncryptionUtil.class.getName()).log(Level.SEVERE, null, ex);
        }
        return null;
    }

    /**
     * AES 解密操作
     *
     * @param content
     * @param key
     * @return
     */
    public static String aesDncode(String content, String key) {
        try {
            //实例化
            Cipher cipher = Cipher.getInstance(DEFAULT_CIPHER_ALGORITHM);
            //使用密钥初始化，设置为解密模式
            cipher.init(Cipher.DECRYPT_MODE, getSecretKey(key));
            //执行操作
            byte[] result = cipher.doFinal(Base64Utils.decode(content.getBytes()));
            return new String(result, "utf-8");
        } catch (Exception ex) {
            Logger.getLogger(EncryptionUtil.class.getName()).log(Level.SEVERE, null, ex);
        }
        return null;
    }

    /**
     * 生成加密秘钥
     *
     * @return
     */
    private static SecretKeySpec getSecretKey(final String key) {
        //返回生成指定算法密钥生成器的 KeyGenerator 对象
        KeyGenerator kg = null;
        try {
            kg = KeyGenerator.getInstance(KEY_ALGORITHM);
            //AES 要求密钥长度为 128
            SecureRandom secureRandom = SecureRandom.getInstance("SHA1PRNG");
            secureRandom.setSeed(key.getBytes());
            kg.init(128, secureRandom);
            //生成一个密钥
            SecretKey secretKey = kg.generateKey();
            // 转换为AES专用密钥
            return new SecretKeySpec(secretKey.getEncoded(), KEY_ALGORITHM);
        } catch (NoSuchAlgorithmException ex) {
            Logger.getLogger(EncryptionUtil.class.getName()).log(Level.SEVERE, null, ex);
        }
        return null;
    }

    public static void main(String[] args) {
        try {
            EncryptionUtil se = new EncryptionUtil();
            Scanner scanner = new Scanner(System.in);
            //加密
            System.out.println("请输入加密的KEY值:");
            String key = scanner.next();
            System.out.println("请输入客户信息:");
            String customer = scanner.next();
            System.out.println("请输入授权时长（天）:");
            String duration = scanner.next();
            System.out.println("请输入授权功能:");
            String function = scanner.next();
            System.out.println(MacAdressUtil.getMacAdress());
            String macadress = MacAdressUtil.getMacAdress().get(0);
            Date dueDate  = DateTimeUtil.getBehindDay(new Date(), Integer.parseInt(duration));
            String content = customer + ";" + DateTimeUtil.dateToStr(dueDate) + ";" + function + ";" + macadress;
            String aesEncode = aesEncode(content, key);
            System.out.println("根据输入的规则" + key + "加密后的密文是:" + aesEncode(content, key));

            //解密
            System.out.println("使用AES对称解密，请输入加密的规则：(须与加密相同)");
            key = scanner.next();
            System.out.println("请输入要解密的内容（密文）:");
            content = scanner.next();
            System.out.println("根据输入的规则" + key + "解密后的明文是:" + aesDncode(content, key));
            File file = new File("D:/winsw/license.txt");
            //文件不存在，创建新的文件
            if (!file.exists()) {
                file.createNewFile();
            }
            FileWriter fileWriter = new FileWriter(file);
            fileWriter.write(aesEncode);
            fileWriter.flush();
            fileWriter.close();
        } catch (SocketException e){
            System.out.println("获取mac地址失败");
        } catch (IOException e){
            System.out.println("生成license文件失败");
        }
    }
}
