package com.nationrel.modules.rga.general.util.common;

import java.lang.reflect.Field;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author HuangKang on 2018/11/28.
 */
public class ObjectActionUtil {
    public static Object emptyObject = null;

    /**
     * 判断对象为空
     *
     * @param obj 对象名
     * @return 是否为空
     */
    @SuppressWarnings("rawtypes")
    public static boolean isEmpty(Object obj) {
        if (obj == null) {
            return true;
        }
        if ((obj instanceof List)) {
            return ((List) obj).size() == 0;
        }
        if ((obj instanceof String)) {
            return "".equals(((String) obj).trim());
        }
        return false;
    }

    /**
     * 判断对象不为空
     *
     * @param obj 对象名
     * @return 是否不为空
     */
    public static boolean isNotEmpty(Object obj) {
        return !isEmpty(obj);
    }

    /***
     * Determine whether the object's fields are empty
     *
     * @param obj
     * @param isExcludeZero  :true:数值类型的值为0,则当做为空;----false:数值类型的值为0,则不为空
     *
     * @return
     * @throws SecurityException
     * @throws IllegalArgumentException
     * @throws NoSuchFieldException
     * @throws IllegalAccessException
     */
    public static boolean isNullObject(Object obj, boolean isExcludeZero) {
        List<Field> fieldList = ReflectHWUtil.getAllFieldList(obj.getClass());
        boolean isNull = true;
        for (int i = 0; i < fieldList.size(); i++) {
            Field f = fieldList.get(i);
            // 设置对象的访问权限，保证对private的属性的访问
            Object propertyValue = getObjectValue(obj, f);

            if (!ValueWidgetUtil.isNullOrEmpty(propertyValue)) {// 字段不为空
                if (propertyValue instanceof Integer) {// 是数字
                    if (!((Integer) propertyValue == 0 && isExcludeZero)) {
                        isNull = false;
                        break;
                    }
                } else if (propertyValue instanceof Double) {// 是数字
                    if (!((Double) propertyValue == 0 && isExcludeZero)) {
                        isNull = false;
                        break;
                    }
                } else if (propertyValue instanceof Float) {// 是数字
                    if (!((Float) propertyValue == 0 && isExcludeZero)) {
                        isNull = false;
                        break;
                    }
                } else if (propertyValue instanceof Short) {// 是数字
                    if (!((Short) propertyValue == 0 && isExcludeZero)) {
                        isNull = false;
                        break;
                    }
                } else {
                    isNull = false;
                    break;
                }
            }
        }
        return isNull;
    }

    /***
     * 获取指定对象的属性值
     *
     * @param obj
     * @param name
     */
    public static Object getObjectValue(Object obj, Field name) {
        if (name == null) {
            return null;
        }
        name.setAccessible(true);
        try {
            return name.get(obj);
        } catch (IllegalAccessException e) {
            e.printStackTrace();
            return null;
        }
    }

    /**
     * 将实体类转为Map
     *
     * @param model
     * @return
     * @throws Exception
     */
    public static Map<String, Object> reflect(Object model) {
        Map<String, Object> result = new HashMap<>();
        try {
            for (Field field : model.getClass().getDeclaredFields()) {
                field.setAccessible(true);
                result.put(field.getName(), field.get(model));
            }
            return result;
        } catch (Exception e) {
            e.printStackTrace();
            return null;
        }
    }
}
