package com.nationrel.modules.rga.general.util.common;

import org.springframework.beans.BeansException;
import org.springframework.context.ApplicationContext;
import org.springframework.context.ApplicationContextAware;

/**
 * @ClassName SpringContextUtil
 * @Description Spring的上下文工具
 * @version V1.0
 */
public class SpringContextUtil implements ApplicationContextAware {

    /** Spring的上下文环境 */
    private static ApplicationContext applicationContext;

    @Override
    public void setApplicationContext(ApplicationContext applicationContext)
        throws BeansException {
        SpringContextUtil.applicationContext = applicationContext;
    }

    /**
     * @return the applicationContext
     */
    public static ApplicationContext getApplicationContext() {
        return applicationContext;
    }

    /**
     * @Title: getBean
     * @Description: 以class类型取得注入bean
     * @param c
     * @return T
     */
    @SuppressWarnings("unchecked")
    public static <T> T getBean(Class<T> c) {
        String[] names = applicationContext.getBeanNamesForType(c);
        if (names.length > 0) {
            return (T) applicationContext.getBean(names[0]);
        }
        return null;
    }

    /**
     * @Title: getBean
     * @Description: 取得Bean
     * @param beanName
     * @return Object
     */
    public static Object getBean(String beanName) {
        Object bean = null;
        try {
            bean = applicationContext.getBean(beanName);
        } catch (BeansException e) {
            // Nothing
        }
        return bean;
    }
}
