package com.nationrel.modules.rga.util;

import com.nationrel.modules.rga.constant.RgaConstants;
import com.nationrel.modules.rga.general.domain.enums.reliaqube.ReliaqubeParamDataCheckEnum;
import com.nationrel.modules.rga.general.exception.DescribeException;
import org.apache.poi.ss.util.NumberToTextConverter;
import org.apache.poi.xssf.usermodel.XSSFCell;
import org.apache.poi.xssf.usermodel.XSSFRow;
import org.apache.poi.xssf.usermodel.XSSFSheet;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;

import java.io.InputStream;
import java.text.DecimalFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.List;

/**
 * 导入Excel工具类
 */
public class ImportExcelUtil {

    /**
     * 获取IO流中的数据，组装成List<List<Object>>对象
     * @param inStr
     * @param fileName
     * @return
     * @throws Exception
     */
    public static List<List<Object>> getListByExcel(InputStream inStr, String fileName, Integer languageKey) throws Exception {
        List<List<Object>> list = null;

        //创建Excel工作薄
        XSSFWorkbook wb = getWorkbook(inStr, fileName, languageKey);
        XSSFSheet sheet = null;
        XSSFRow row = null;
        XSSFCell cell = null;

        list = new ArrayList<>();
        //遍历Excel中所有的sheet
        for (int i = 0; i < wb.getNumberOfSheets(); i++) {
            sheet = wb.getSheetAt(i);
            if (sheet == null) {
                continue;
            }
            //遍历当前sheet中的所有行
            for (int j = sheet.getFirstRowNum(); j < sheet.getLastRowNum() + 1; j++) {
                row = sheet.getRow(j);
                if (row == null) {
                    continue;
                }
                //遍历所有的列
                List<Object> li = new ArrayList<>();
                for (int y = row.getFirstCellNum(); y < row.getLastCellNum(); y++) {
                    cell = row.getCell(y);
                    li.add(getCellValue(cell));
                }
                list.add(li);
            }
        }
        return list;
    }

    /**
     * 根据文件后缀，自适应上传文件的版本
     * @param inStr
     * @param fileName
     * @return
     * @throws Exception
     */
    public static XSSFWorkbook getWorkbook(InputStream inStr, String fileName, Integer languageKey) throws Exception {
        XSSFWorkbook wb = null;
        if (fileName.toLowerCase().contains(RgaConstants.FILE_TYPE_XLSX)) {
            wb = new XSSFWorkbook(inStr);
        } else {
            throw new DescribeException(ReliaqubeParamDataCheckEnum.请上传正确的文件格式xlsx, languageKey);
        }
        return wb;
    }

    /**
     * 对表格中数值进行格式化
     * @param cell
     * @return
     */
    public static Object getCellValue(XSSFCell cell) {
        Object value = null;
        DecimalFormat df = new DecimalFormat("0");  //格式化number String字符
        SimpleDateFormat sdf = new SimpleDateFormat("yyy-MM-dd");  //日期格式化
        DecimalFormat df2 = new DecimalFormat("0.00");  //格式化数字

        switch (cell.getCellType()) {
            case STRING:
                value = cell.getRichStringCellValue().getString();
                break;
            case NUMERIC:
                if ("General".equals(cell.getCellStyle().getDataFormatString())) {
                    value = NumberToTextConverter.toText(cell.getNumericCellValue()); // 解决精度丢失问题
                } else if ("m/d/yy".equals(cell.getCellStyle().getDataFormatString())) {
                    value = sdf.format(cell.getDateCellValue());
                } else {
                    value = df2.format((int) cell.getNumericCellValue());
                }
                break;
            case BOOLEAN:
                value = cell.getBooleanCellValue();
                break;
            case BLANK:
                value = "";
                break;
            default:
                break;
        }
        return value;
    }
}
