package com.nationrel.modules.rga.util;

import com.nationrel.modules.rga.constant.RgaConstants;
import com.nationrel.modules.rga.entity.RgaData;
import org.springframework.stereotype.Component;

import java.lang.reflect.Field;
import java.sql.*;
import java.util.List;

@Component
public class JDBC2DataUtil {

    public static void batchInsert(List<RgaData> rgaData, String mysqlUrl, String mysqlUserName, String mysqlPassword) {
        //1.导入驱动jar包
        //2.注册驱动(mysql5之后的驱动jar包可以省略注册驱动的步骤)
        //Class.forName("com.mysql.jdbc.Driver");
        //3.获取数据库连接对象
        Connection conn = null;
        PreparedStatement pstmt = null;
        try {
            //"&rewriteBatchedStatements=true",一次插入多条数据，只插入一次
            conn = DriverManager.getConnection(mysqlUrl.split(" \\?")[0] + "&rewriteBatchedStatements=true", mysqlUserName, mysqlPassword);
            conn.setAutoCommit(false);
            String sql = "delete from rga_data where analysis_id = " + "'" + rgaData.get(0).getAnalysisId() + "'";
            pstmt = conn.prepareStatement(sql);
            pstmt.execute();
            //4.定义sql语句
            sql = "insert into rga_data (ANALYSIS_ID,DATA_NO,QUANTITY,LAST_INSPECT_TIME,F_S,FAILURE_END_TIME,COMMENT1,COMMENT2,COMMENT3,COMMENT4,COMMENT5,COMMENT6,COMMENT7,COMMENT8) values(?,?,?,?,?,?,?,?,?,?,?,?,?,?)";
            //5.获取执行sql的对象PreparedStatement
            pstmt = conn.prepareStatement(sql);
            int resultNum = 0;
            //6.不断产生sql
            for (int i = 0, size = rgaData.size(); i < size; i++) {
                RgaData dataDetail = rgaData.get(i);
                if (i != 0 && i % RgaConstants.MAX_BATCH_NUMMAX == 0) {
                    pstmt.executeBatch();
                    pstmt.clearBatch();
                    resultNum = 0;
                }
                resultNum++;
                pstmt.setString(1, dataDetail.getAnalysisId());
                if (dataDetail.getDataNo() == null) {
                    pstmt.setNull(2, Types.INTEGER);
                } else {
                    pstmt.setInt(2, dataDetail.getDataNo());
                }
                if (dataDetail.getQuantity() == null) {
                    pstmt.setNull(3, Types.INTEGER);
                } else {
                    pstmt.setInt(3, dataDetail.getQuantity());
                }
                if (dataDetail.getLastInspectTime() == null) {
                    pstmt.setNull(4, Types.DOUBLE);
                } else {
                    pstmt.setDouble(4, dataDetail.getLastInspectTime());
                }
                pstmt.setString(5, dataDetail.getFailureSuspension());
                if (dataDetail.getFailureEndTime() == null) {
                    pstmt.setNull(6, Types.DOUBLE);
                } else {
                    pstmt.setDouble(6, dataDetail.getFailureEndTime());
                }
                // 通过反射拼接sql
                handleComment(pstmt, dataDetail);
                pstmt.addBatch();
            }
            //7.往数据库插入一次数据
            if (resultNum > 0) {
                pstmt.executeBatch();
            }
            conn.commit();//执行完后，手动提交事务
            conn.setAutoCommit(true);//再把自动提交打开，避免影响其他需要自动提交的操作
        } catch (SQLException e) {
            e.printStackTrace();
        } catch (NoSuchFieldException e) {
            e.printStackTrace();
        } catch (IllegalAccessException e) {
            e.printStackTrace();
        } finally {
            //8.释放资源
            //避免空指针异常
            if (pstmt != null) {
                try {
                    pstmt.close();
                } catch (SQLException e) {
                    e.printStackTrace();
                }
            }

            if (conn != null) {
                try {
                    conn.close();
                } catch (SQLException e) {
                    e.printStackTrace();
                }
            }
        }
    }

    public static void handleComment(PreparedStatement pstmt, RgaData dataDetail) throws NoSuchFieldException, IllegalAccessException, SQLException {
        // 1.获取AltaData类对应的字节码对象
        Class<?> clazz = dataDetail.getClass();
        //注意！目前成员变量的修饰符必须是public的才能获取到
        for (int i = 1; i < 9; i++) {
            // 2.通过AltaData类对应的字节码对象获取AltaData类中的成员变量们
            Field field = clazz.getDeclaredField("comment" + i);
            // 3.设置对象的访问权限，保证对private的属性的访问
            field.setAccessible(true);
            System.err.println(field.get(dataDetail));
            if (field.get(dataDetail) != null) {
                pstmt.setDouble(i + 6, (Double)field.get(dataDetail));
            } else {
                pstmt.setNull(i + 6, Types.DOUBLE);
            }
        }
    }
}
