package com.nationrel.modules.rga.service;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.nationrel.modules.rga.config.AllVariableConfig;
import com.nationrel.modules.rga.mapper.*;
import com.nationrel.modules.rga.constant.RgaConstants;
import com.nationrel.modules.rga.dto.RgaAnalysisDataSaveDto;
import com.nationrel.modules.rga.dto.RgaColumnsEditDto;
import com.nationrel.modules.rga.entity.*;
import com.nationrel.modules.rga.enums.RgaDistributingCodeEnum;
import com.nationrel.modules.rga.enums.RgaLifeStressModelEnum;
import com.nationrel.modules.rga.util.JDBC2DataUtil;
import com.nationrel.modules.rga.vo.RgaAnalysisSetDataVo;
import com.nationrel.modules.rga.vo.RgaCalcResultVo;
import com.nationrel.modules.rga.vo.RgaColumnVo;
import com.nationrel.modules.rga.general.common.ServerResponse;
import com.nationrel.modules.rga.general.domain.auth.HeaderUserInfo;
import com.nationrel.modules.rga.general.domain.enums.alta.AltaCustomPromptEnum;
import com.nationrel.modules.rga.general.domain.enums.common.EwbConstants;
import com.nationrel.modules.rga.general.domain.enums.reliaqube.ReliaqubeParamDataCheckEnum;
import com.nationrel.modules.rga.general.domain.enums.reliaqube.ReliaqubeProgramExceptionEnum;
import com.nationrel.modules.rga.general.domain.enums.weibull.EwbCommonUsedEnum;
import com.nationrel.modules.rga.general.domain.enums.weibull.EwbCustomPromptEnum;
import com.nationrel.modules.rga.general.exception.DescribeException;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import com.nationrel.modules.rga.util.NumberParseUtil;

import java.util.*;

@Slf4j
@Service
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class RgaColumnAndDataService {

    @Value("${spring.datasource.dynamic.datasource.master.url}")
    private String mysqlUrl;
    @Value("${spring.datasource.dynamic.datasource.master.username}")
    private String mysqlUserName;
    @Value("${spring.datasource.dynamic.datasource.master.password}")
    private String mysqlPassword;
    private final AllVariableConfig allVariableConfig;
    private final RgaDataMapper dataMapper;
    private final RgaPlotMapper plotMapper;
    private final RgaColumnMapper columnMapper;
    private final RgaSettingMapper settingMapper;
    private final RgaCalcResultMapper calcResultMapper;
    private final RgaValidatorService validatorService;
    private final RgaAnalysisMapper analysisMapper;
    private final RgaProjectMapper projectMapper;

    /**
     * 分页查询数据分析数据
     *
     * @param analysisId
     * @param pageIndex
     * @param pageSize
     * @return
     */
    public ServerResponse<RgaAnalysisSetDataVo> listDataByPage(String analysisId, int pageIndex, int pageSize) {
        RgaSetting setting = settingMapper.selectById(analysisId);
        List<RgaColumn> columns = columnMapper.selectList(new QueryWrapper<RgaColumn>().lambda()
                .eq(RgaColumn::getAnalysisId, analysisId).eq(RgaColumn::getSelectStatus, 1));
        List<RgaColumnVo> columnVos = new ArrayList<>(columns.size());
        columns.forEach(column -> {
            column.setName(column.getName() + "(" + column.getUnit() + ")");
            RgaColumnVo columnVo = new RgaColumnVo();
            BeanUtils.copyProperties(column, columnVo);
            columnVos.add(columnVo);
        });
        // 创建基础查询条件
        List<RgaData> list = dataMapper.selectList(new QueryWrapper<RgaData>().lambda().eq(RgaData::getAnalysisId, analysisId));
        // 简化数据以减少网络数据传输
        List<String[]> datas = new ArrayList<>(list.size());
        list.forEach(data -> datas.add(RgaData.toStringStr(data)));
        return ServerResponse.createBySuccess(new RgaAnalysisSetDataVo(setting, datas, columnVos, analysisMapper.selectById(analysisId).getEndTimeName(), allVariableConfig.getAltaDataNumber()));
    }

    /**
     * 获取计算结果数据
     *
     * @param analysisId
     * @return
     */
    public ServerResponse<List<RgaCalcResultVo>> getCalcResult(String analysisId) {
        RgaSetting setting = settingMapper.selectById(analysisId);
        RgaCalcResult calcResult = calcResultMapper.selectOne(
                new QueryWrapper<RgaCalcResult>().lambda().eq(RgaCalcResult::getAnalysisId, setting.getAnalysisId())
                        .eq(RgaCalcResult::getLifeStressModel, setting.getLifeStressModel())
                        .eq(RgaCalcResult::getDistributionType, setting.getDistributionType()));
        return ServerResponse.createBySuccess(RgaCalcResultService.getcalcResultVos(setting.getLifeStressModel(),
                setting.getDistributionType(), calcResult));
    }

    /**
     * 查询加速数据分析应力设置
     *
     * @param analysisId
     * @return
     */
    public ServerResponse<List<RgaColumn>> listColumn(String analysisId, Integer languageKey) {
        QueryWrapper<RgaColumn> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(RgaColumn::getAnalysisId, analysisId);
        return ServerResponse.createBySuccess(columnMapper.selectList(queryWrapper));

    }

    /**
     * 修改应力项
     *
     * @param entity
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    public ServerResponse<String> updateColumns(HeaderUserInfo userInfo, RgaColumnsEditDto entity) {
        if (CollectionUtils.isEmpty(entity.getSelectColumns())){
            return ServerResponse.createByErrorCodeMessage(ReliaqubeProgramExceptionEnum.ParamterDataException.getIndex(), AltaCustomPromptEnum.getLanguageMessageByKey(AltaCustomPromptEnum.至少选择一个应力, userInfo.getLanguageKey()));
        }
        // 删除原有应力项配置
        columnMapper.delete(new QueryWrapper<RgaColumn>().lambda().eq(RgaColumn::getAnalysisId,
                entity.getAllColumns().get(0).getAnalysisId()));
        // 获取全部的应力项
        List<RgaColumn> result = entity.getAllColumns();
        if (CollectionUtils.isEmpty(entity.getSelectColumns())) {
            columnMapper.insertBatch(result);
        } else {
            // 处理选中的应力项
            for (RgaColumn column : result) {
                column.setSelectStatus(0);
                for (RgaColumn selectColumn : entity.getSelectColumns()) {
                    if (column.getColumnId().equals(selectColumn.getColumnId())) {
                        column.setSelectStatus(1);
                    }
                }
            }
            columnMapper.insertBatch(result);
        }
        // 删除所有的echart图信息
        plotMapper.delete(new QueryWrapper<RgaPlot>().lambda().eq(RgaPlot::getAnalysisId, entity.getAllColumns().get(0).getAnalysisId()));
        RgaProjectService.updateUpdateInfo(userInfo, analysisMapper.selectById(result.get(0).getAnalysisId()).getProjectId(), projectMapper);
        return ServerResponse.createBySuccess();
    }

    /**
     * 保存数据
     *
     * @param userInfo
     * @param entity
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    public ServerResponse<String> updateData(HeaderUserInfo userInfo, RgaAnalysisDataSaveDto entity) {
        String analysisId = entity.getAnalysisId();
        // ==================
        // 1.修改寿命数据分析数据表信息
        // ==================
        RgaSetting setting = settingMapper.selectById(analysisId);
        boolean hasChangeSetting = !setting.getRightFlag().equals(entity.getRightFlag()) || !setting.getLeftFlag().equals(entity.getLeftFlag()) || !setting.getGroupingFlag().equals(entity.getGroupingFlag());
        // 包含删失数据(右删失数据)
        if ("0".equals(entity.getRightFlag()) && !"0".equals(setting.getRightFlag())) {
            // 开关关闭，则需要将所有数据的F/S置为S
            dataMapper.updatefByAnalysisId(analysisId);
        }
        // 包含删失数据(左删失数据)
        if ("0".equals(entity.getLeftFlag()) && !"0".equals(setting.getLeftFlag())) {
            // 开关关闭，则需要将所有数据的最近检查时间置为空
            dataMapper.updateLitByAnalysisId(analysisId);
        }
        // 分组数据关闭
        if ("0".equals(entity.getGroupingFlag()) && !"0".equals(setting.getGroupingFlag())) {
            // 开关关闭，则需要将所有数据的数量置为1
            dataMapper.updateQuantityByAnalysisId(analysisId);
        }
        // 删除原有的计算结果以及画图设置
        deleteAnalysisResultData(setting.getAnalysisId(), calcResultMapper, settingMapper);
        if (hasChangeSetting) {
            setting.setLeftFlag(entity.getLeftFlag());
            setting.setRightFlag(entity.getRightFlag());
            setting.setUpdateBy(userInfo.getUserName());
            setting.setGroupingFlag(entity.getGroupingFlag());
            setting.setUpdateTime(new Date());
            // 寿命数据分析数据表修改
            settingMapper.updateById(setting);
        }
        if (entity.getHotDataChange() > 1) {
            List<RgaData> analysisDataDetails = new ArrayList<>();
            // 解析新的数组结构的数据，将其解析为对象
            for (int i = 0; i < entity.getAnalysisDatas().size(); i++) {
                String[] data = entity.getAnalysisDatas().get(i);
                if (data[0] != null && data[1] != null) {
                    // 判断填入数值是否为大于0的数字
                    if (!NumberParseUtil.isNumeric(data[4]) || NumberParseUtil.parseDouble(data[4]) <= 0) {
                        throw new DescribeException((i + 1), EwbCommonUsedEnum.getLanguageMessageByKey(EwbCommonUsedEnum.结束时间, userInfo.getLanguageKey()), EwbCustomPromptEnum.必须为大于零数字, userInfo.getLanguageKey());
                    }
                    // 判断最近检查时间填入数值是否为大于0的数字
                    if (entity.getLeftFlag().equals(EwbConstants.FLAG_CHECKED) && (!NumberParseUtil.isNumeric(data[2]) || NumberParseUtil.parseDouble(data[2]) <= 0)) {
                        throw new DescribeException((i + 1), EwbCommonUsedEnum.getLanguageMessageByKey(EwbCommonUsedEnum.最近检查时间, userInfo.getLanguageKey()), EwbCustomPromptEnum.必须为大于零数字, userInfo.getLanguageKey());
                    }
                    // 判断最近检查时间是否合法
                    if (entity.getLeftFlag().equals(EwbConstants.FLAG_CHECKED) && NumberParseUtil.parseDouble(data[2]) > NumberParseUtil.parseDouble(data[4])) {
                        throw new DescribeException((i + 1), ReliaqubeParamDataCheckEnum.结束时间必须大于等于最近检查时间, userInfo.getLanguageKey());
                    }
                    analysisDataDetails.add(RgaData.toAltaData(data, entity.getAnalysisId()));
                }
            }
            validatorService.validatorAnalysisDetailData(analysisDataDetails, entity.getGroupingFlag(), entity.getLeftFlag(), entity.getRightFlag(), entity.getStartDataNo(), userInfo.getLanguageKey());
            saveData(analysisId, analysisDataDetails, entity.getLeftFlag(), dataMapper, plotMapper, calcResultMapper, userInfo.getLanguageKey());
            projectMapper.updateUpdateInfo(new Date(), userInfo.getUserName(), entity.getAltaId());
        }
        // 返回结果
        return ServerResponse.createBySuccess();
    }

    /**
     * 删除中间数据
     *
     * @param analysisId
     */
    public static void deleteAnalysisResultData(String analysisId, RgaCalcResultMapper calcResultMapper, RgaSettingMapper settingMapper) {
        // 删除计算结果数据
        calcResultMapper.deleteByAnalysisId(analysisId);
//        settingMapper.deleteAllCalcResult(settingId);
        // 删除xy区间信息
        settingMapper.deleteMarkSetting(analysisId);
    }

    @Transactional(rollbackFor = Exception.class)
    public void saveData(String analysisId, List<RgaData> dataList, String leftFlag, RgaDataMapper dataMapper, RgaPlotMapper plotMapper, RgaCalcResultMapper calcResultMapper, Integer languageKey) {
        if (dataList != null && dataList.size() > 0) {
            // 删除集合中WeiMaterial对象属性fratios为空的对象s
            dataList.removeIf(weiMaterial -> weiMaterial.getFailureEndTime() == null);
        }

        // 插入起始数据编号
        int newStartDataNo = RgaConstants.NUM_ONE;
        if (dataList != null && dataList.size() > 0) {
            // ==================
            // 3.保存寿命数据分析明细数据
            // ==================
            // 分析明细数据处理
            List<RgaData> dataDetails = new ArrayList<>();
            for (RgaData dataDetail : dataList) {
                if (dataDetail != null) {
                    if (RgaConstants.FLAG_CHECKED.equals(leftFlag) && dataDetail.getLastInspectTime() == null) {
                        throw new DescribeException(newStartDataNo, ReliaqubeParamDataCheckEnum.最近检查时间不可为空, languageKey);
                    }
                    String fs = dataDetail.getFailureSuspension();
                    // ==================
                    // 判断明细数据的数据类型
                    // ==================
                    if (RgaConstants.STATE_SUSPENSION.equalsIgnoreCase(fs)) {
                        // 若状态为s，则数据类型为右删失数据类型
                        dataDetail.setFailureSuspension(RgaConstants.STATE_S);
                        // dataDetail.setFailureEndTime(dataDetail.getFailureEndTime());
                    } else {
                        dataDetail.setFailureSuspension(RgaConstants.STATE_F);
                        // 对于左删失，区间，完整数据，需要计算平均时间
                        // dataDetail.setFailureEndTime((dataDetail.getFailureEndTime() +
                        // lastInspectTime) / 2);
                    }
                    if (null == dataDetail.getQuantity()) {
                        dataDetail.setQuantity(1);
                    } else if (dataDetail.getQuantity().equals(0)) {
                        throw new DescribeException(newStartDataNo, ReliaqubeParamDataCheckEnum.数量不可为空, languageKey);
                    }
                    // 数据编号
                    dataDetail.setDataNo(newStartDataNo);
                    newStartDataNo++;
                    // 数据ID
                    dataDetail.setAnalysisId(analysisId);
                    dataDetails.add(dataDetail);
                }
            }
            // ==================
            // 1.删除当前页面分析明细数据表数据
            // ==================
            // 删除所有的echart图信息
            plotMapper.delete(new QueryWrapper<RgaPlot>().lambda().eq(RgaPlot::getAnalysisId, analysisId));
            // 删除所有的计算信息
            calcResultMapper.delete(new QueryWrapper<RgaCalcResult>().lambda().eq(RgaCalcResult::getAnalysisId, analysisId));
            JDBC2DataUtil.batchInsert(dataDetails, mysqlUrl, mysqlUserName, mysqlPassword);
        }
    }

    public ServerResponse<Map> listSelectColumn(String analysisId, Integer languageKey) {
        Map map = new HashMap();
        List<RgaColumn> columns = columnMapper.selectList(new QueryWrapper<RgaColumn>().lambda().eq(RgaColumn::getAnalysisId, analysisId).eq(RgaColumn::getShowStatus, 1));
        map.put("column", columns);
        RgaSetting setting = settingMapper.selectById(analysisId);
        String analysisSetting = RgaLifeStressModelEnum.getNameByLanguageKey(setting.getLifeStressModel().toString(), languageKey) + "; " + RgaDistributingCodeEnum.getNameByLanguageKey(setting.getDistributionType().toString(), languageKey) + "; ";
        for (RgaColumn column : columns) {
            analysisSetting = analysisSetting + column.getName() + "(" + column.getUnit() + ")" + ":" + column.getStressLevel() + "; ";
        }
        map.put("analysisSetting", analysisSetting);
        return ServerResponse.createBySuccess(map);
    }
}
