// Copyright(c) 2021 ReliaQube. All rights reserved.
package com.nationrel.modules.rga.service;

import com.alibaba.fastjson.JSONObject;
import com.nationrel.modules.rga.config.AllVariableConfig;
import com.nationrel.modules.rga.constant.*;
import com.nationrel.modules.rga.dto.RgaFastCalculateDto;
import com.nationrel.modules.rga.entity.RgaColumn;
import com.nationrel.modules.rga.entity.RgaData;
import com.nationrel.modules.rga.entity.RgaSetting;
import com.nationrel.modules.rga.enums.RgaCalcParametersEnum;
import com.nationrel.modules.rga.general.domain.enums.reliaqube.ReliaqubeParamDataCheckEnum;
import com.nationrel.modules.rga.general.exception.DescribeException;
import com.nationrel.modules.rga.general.util.common.BasicDataUtil;
import com.nationrel.modules.rga.model.RgaCalcRequestModel;
import lombok.RequiredArgsConstructor;
import org.apache.commons.httpclient.HttpClient;
import org.apache.commons.httpclient.methods.PostMethod;
import org.apache.commons.httpclient.methods.RequestEntity;
import org.apache.commons.httpclient.methods.StringRequestEntity;
import org.apache.commons.lang3.StringUtils;
import org.apache.http.params.CoreConnectionPNames;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.BufferedReader;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.SocketTimeoutException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @version V1.0
 * @ClassName RldAnalysisChartDataServiceImpl
 * @Description api调用
 * @Author aaron
 * @CreateDate 2018-11-1
 */
@SuppressWarnings("AliDeprecation")
@Service
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class RgaRgaApiService {

    private final AllVariableConfig allVariableConfig;
    private int retry;

    public String getResultsCallApi(JSONObject requestData, String url, String contentType, String acceptType) {
        PostMethod postMethod = null;
        String responseResult = "";
        try {
            HttpClient clientPlot = new HttpClient();
            postMethod = new PostMethod(url);
            RequestEntity requestEntityPlot = new StringRequestEntity(
                    requestData.toString(), "text/xml", "UTF-8");
            postMethod.setRequestEntity(requestEntityPlot);
            postMethod.setRequestHeader("Content-Type", contentType);
            if (StringUtils.isNotEmpty(acceptType)) {
                postMethod.setRequestHeader("Accept", acceptType);
            }
            clientPlot.executeMethod(postMethod);
            responseResult = new String(postMethod.getResponseBodyAsString().getBytes("UTF-8"));
        } catch (Exception e) {
            if (retry < 3) {
                retry++;
                // 如果连接失效，重试
                getResultsCallApi(requestData, url, contentType, acceptType);
            } else {
                retry = 0;
                e.printStackTrace();
                //throw new ApplicationRuntimeException("无法链接寿命数据算法服务器");
            }
        } finally {
            postMethod.releaseConnection();
        }
        System.err.println("responseResult => " + responseResult);
        return responseResult;
    }

    public String getResultsCallApi(JSONObject requestData, String url, String contentType, String acceptType, String body) {
        PostMethod postMethod = null;
        String responseResult = "";
        try {
            HttpClient clientPlot = new HttpClient();
            System.err.println(body);
            System.err.println(url);
            postMethod = new PostMethod(url);
            RequestEntity requestEntityPlot = new StringRequestEntity(requestData.toString(), "text/xml", "UTF-8");
            postMethod.setRequestEntity(requestEntityPlot);
            postMethod.setRequestHeader("Content-Type", contentType);
            if (StringUtils.isNotEmpty(acceptType)) {
                postMethod.setRequestHeader("Accept", acceptType);
            }

            postMethod.setRequestBody(RgaConstants.SERVER_SYSTEM_LINUX.equals(allVariableConfig.getServerSystem()) ? body : body.replaceAll("\\\":\\\"\\[", "\":[").replaceAll("]\",\"", "],\""));
            clientPlot.getParams().setParameter(CoreConnectionPNames.CONNECTION_TIMEOUT, 5000);
            clientPlot.getParams().setParameter(CoreConnectionPNames.SO_TIMEOUT, 15000);
            clientPlot.executeMethod(postMethod);
//            responseResult = new String(postMethod.getResponseBodyAsString().getBytes("UTF-8"));
            InputStream inputStream = postMethod.getResponseBodyAsStream();
            BufferedReader br = new BufferedReader(new InputStreamReader(inputStream));
            StringBuffer stringBuffer = new StringBuffer();
            String str = "";
            while ((str = br.readLine()) != null) {
                stringBuffer.append(str);
            }
            responseResult = stringBuffer.toString();
        } catch (SocketTimeoutException e) {
            throw new DescribeException(ReliaqubeParamDataCheckEnum.当前数据在该模型下无法进行分析, 1);
        } catch (Exception e) {
            if (retry < 3) {
                retry++;
                // 如果连接失效，重试
                getResultsCallApi(requestData, url, contentType, acceptType, body);
            } else {
                retry = 0;
                e.printStackTrace();
//				throw new Exception("无法链接服务器");
            }
        } finally {
            postMethod.releaseConnection();
        }
        System.err.println("responseResult => " + responseResult);
        return responseResult;
    }


    /**
     * 根据当前的设置生成需要调用的url
     *
     * @param distributionType
     * @param lifeStressModel
     * @return
     */
    public String getAltaUri(Integer distributionType, Integer lifeStressModel) {
        String uri = null;
        if (distributionType.equals(RgaDistributionTypeEnum.Weibull.getIndex())) {
            if (lifeStressModel.equals(RgaLifeStressModelEnum.Arrhenius.getIndex())
                    || lifeStressModel.equals(RgaLifeStressModelEnum.Eyring.getIndex())
                    || lifeStressModel.equals(RgaLifeStressModelEnum.IPL.getIndex())) {
                uri = allVariableConfig.getReliaQubeUrl() + RgaConstants.WEIBULL_1_STRESS + "?LSR=" + RgaLifeStressModelEnum.getName(lifeStressModel);
            } else if (lifeStressModel.equals(RgaLifeStressModelEnum.Temp_Humidity.getIndex())
                    || lifeStressModel.equals(RgaLifeStressModelEnum.Temp_NonThermal.getIndex())) {
                uri = allVariableConfig.getReliaQubeUrl() + RgaConstants.WEIBULL_2_STRESS + "?LSR=" + RgaLifeStressModelEnum.getName(lifeStressModel);
            } else if (lifeStressModel.equals(RgaLifeStressModelEnum.General_Log_Linear.getIndex())) {
                uri = allVariableConfig.getReliaQubeUrl() + RgaConstants.WEIBULL_MULTI_STRESS;
            }
        } else if (distributionType.equals(RgaDistributionTypeEnum.LogNormal.getIndex())) {
            if (lifeStressModel.equals(RgaLifeStressModelEnum.Arrhenius.getIndex())
                    || lifeStressModel.equals(RgaLifeStressModelEnum.Eyring.getIndex())
                    || lifeStressModel.equals(RgaLifeStressModelEnum.IPL.getIndex())) {
                uri = allVariableConfig.getReliaQubeUrl() + RgaConstants.LOG_NORMAL_1_STRESS + "?LSR=" + RgaLifeStressModelEnum.getName(lifeStressModel);
            } else if (lifeStressModel.equals(RgaLifeStressModelEnum.Temp_Humidity.getIndex())
                    || lifeStressModel.equals(RgaLifeStressModelEnum.Temp_NonThermal.getIndex())) {
                uri = allVariableConfig.getReliaQubeUrl() + RgaConstants.LOG_NORMAL_2_STRESS + "?LSR=" + RgaLifeStressModelEnum.getName(lifeStressModel);
            } else if (lifeStressModel.equals(RgaLifeStressModelEnum.General_Log_Linear.getIndex())) {
                uri = allVariableConfig.getReliaQubeUrl() + RgaConstants.LOG_NORMAL_MULTI_STRESS;
            }
        }
        return uri;

    }

    public String getPlotUri(Integer distributionType, Integer plotType) {
        String uri = null;
        if (plotType.equals(RgaPlotTypeEnum.Reliability.getIndex())) {
            uri = allVariableConfig.getReliaQubeUrl() + RgaPlotTypeEnum.Reliability.getPath() + "?distribution=" + RgaDistributionTypeEnum.getName(distributionType);
        } else if (plotType.equals(RgaPlotTypeEnum.UnReliability.getIndex())) {
            uri = allVariableConfig.getReliaQubeUrl() + RgaPlotTypeEnum.UnReliability.getPath() + "?distribution=" + RgaDistributionTypeEnum.getName(distributionType);
        } else if (plotType.equals(RgaPlotTypeEnum.UseLevelProbability.getIndex())) {
            uri = allVariableConfig.getReliaQubeUrl() + RgaPlotTypeEnum.UseLevelProbability.getPath() + "?distribution=" + RgaDistributionTypeEnum.getName(distributionType);
        } else if (plotType.equals(RgaPlotTypeEnum.FailureRate.getIndex())) {
            uri = allVariableConfig.getReliaQubeUrl() + RgaPlotTypeEnum.FailureRate.getPath() + "?distribution=" + RgaDistributionTypeEnum.getName(distributionType);
        } else if (plotType.equals(RgaPlotTypeEnum.pdf.getIndex())) {
            uri = allVariableConfig.getReliaQubeUrl() + RgaPlotTypeEnum.pdf.getPath() + "?distribution=" + RgaDistributionTypeEnum.getName(distributionType);
        } else if (plotType.equals(RgaPlotTypeEnum.AccelerationFactor.getIndex())) {
            uri = allVariableConfig.getReliaQubeUrl() + RgaPlotTypeEnum.AccelerationFactor.getPath() + "?distribution=" + RgaDistributionTypeEnum.getName(distributionType);
        } else if (plotType.equals(RgaPlotTypeEnum.LifeVsStress.getIndex())) {
            uri = allVariableConfig.getReliaQubeUrl() + RgaPlotTypeEnum.LifeVsStress.getPath() + "?distribution=" + RgaDistributionTypeEnum.getName(distributionType);
        } else if (plotType.equals(RgaPlotTypeEnum.TestLevelProbability.getIndex())) {
            uri = allVariableConfig.getReliaQubeUrl() + RgaPlotTypeEnum.TestLevelProbability.getPath() + "?distribution=" + RgaDistributionTypeEnum.getName(distributionType);
        }
        return uri;
    }

    /**
     * 组装调用接口需要的参数
     *
     * @param rgaColumns
     * @param rgaDataList
     * @param rgaSetting
     * @return
     */
    public Map<String, Object> assemblyData(List<RgaColumn> rgaColumns, List<RgaData> rgaDataList, RgaSetting rgaSetting) {
        String serverSystem = allVariableConfig.getServerSystem();
        List<RgaCalcRequestModel> dataRowList = new ArrayList<>();
        for (RgaData rgaData : rgaDataList) {
            RgaCalcRequestModel dataRow = new RgaCalcRequestModel();
            dataRow.setTime(rgaData.getFailureEndTime());
            dataRow.setLastInspectTime(rgaData.getLastInspectTime());
            dataRow.setTimeforRank(0.0);
            if ("1".equals(rgaData.getFailureSuspension())) {
                dataRow.setFsStatus(RgaCalcParametersEnum.getParameter(serverSystem, RgaCalcParametersEnum.failure));
            } else {
                dataRow.setFsStatus(RgaCalcParametersEnum.getParameter(serverSystem, RgaCalcParametersEnum.suspension));
            }
            dataRow.setNumofForS(rgaData.getQuantity());
            dataRow.setFailureSeverity("string");
            dataRow.setFailureMode("string");
            dataRow.setRank(0);
            List orgStressList = new ArrayList();
            for (RgaColumn rgaColumn : rgaColumns) {
                if (rgaColumn.getColumnId().equals(RgaConstants.COLUMN1)) {
                    orgStressList.add(rgaData.getComment1());
                }
                if (rgaColumn.getColumnId().equals(RgaConstants.COLUMN2)) {
                    orgStressList.add(rgaData.getComment2());
                }
                if (rgaColumn.getColumnId().equals(RgaConstants.COLUMN3)) {
                    orgStressList.add(rgaData.getComment3());
                }
                if (rgaColumn.getColumnId().equals(RgaConstants.COLUMN4)) {
                    orgStressList.add(rgaData.getComment4());
                }
                if (rgaColumn.getColumnId().equals(RgaConstants.COLUMN5)) {
                    orgStressList.add(rgaData.getComment5());
                }
                if (rgaColumn.getColumnId().equals(RgaConstants.COLUMN6)) {
                    orgStressList.add(rgaData.getComment6());
                }
                if (rgaColumn.getColumnId().equals(RgaConstants.COLUMN7)) {
                    orgStressList.add(rgaData.getComment7());
                }
                if (rgaColumn.getColumnId().equals(RgaConstants.COLUMN8)) {
                    orgStressList.add(rgaData.getComment8());
                }
            }
            dataRow.setOrgStress(orgStressList);
            dataRow.setStressGroupId(0);
            dataRow.setOrignialId(0);
            dataRowList.add(dataRow);
        }
        Map<String, Object> requestMap = new HashMap<>();
        requestMap.put(RgaCalcParametersEnum.getParameter(serverSystem, RgaCalcParametersEnum.dataRow), dataRowList);
        requestMap.put(RgaCalcParametersEnum.getParameter(serverSystem, RgaCalcParametersEnum.timeSorted), false);
        requestMap.put(RgaCalcParametersEnum.getParameter(serverSystem, RgaCalcParametersEnum.stressSorted), false);
        requestMap.put(RgaCalcParametersEnum.getParameter(serverSystem, RgaCalcParametersEnum.rankTimeSorted), false);
        requestMap.put(RgaCalcParametersEnum.getParameter(serverSystem, RgaCalcParametersEnum.pointsXYSorted), false);
        requestMap.put(RgaCalcParametersEnum.getParameter(serverSystem, RgaCalcParametersEnum.rankTimeCalculated), false);
        requestMap.put(RgaCalcParametersEnum.getParameter(serverSystem, RgaCalcParametersEnum.totalNumStress), rgaColumns.size());
        requestMap.put(RgaCalcParametersEnum.getParameter(serverSystem, RgaCalcParametersEnum.endTime), 0);
        requestMap.put(RgaCalcParametersEnum.getParameter(serverSystem, RgaCalcParametersEnum.totalNumFailures), 0);
        requestMap.put(RgaCalcParametersEnum.getParameter(serverSystem, RgaCalcParametersEnum.totalNumSuspensions), 0);
        requestMap.put(RgaCalcParametersEnum.getParameter(serverSystem, RgaCalcParametersEnum.totalNumFailureandSuspensions), 0);
        requestMap.put(RgaCalcParametersEnum.getParameter(serverSystem, RgaCalcParametersEnum.totalFailureRows), 0);
        requestMap.put(RgaCalcParametersEnum.getParameter(serverSystem, RgaCalcParametersEnum.totalSuspensionRows), 0);
        requestMap.put(RgaCalcParametersEnum.getParameter(serverSystem, RgaCalcParametersEnum.totalNumofRows), 0);
        // 是否包含S
        boolean s = false;
        for (RgaData rgaData : rgaDataList) {
            if ("0".equals(rgaData.getFailureSuspension())) {
                s = true;
                break;
            }
        }

        // A = 右删失
        // B = 左删失
        if (rgaSetting.getRightFlag().equals(RgaConstants.FLAG_CHECKED) && rgaSetting.getLeftFlag().equals(RgaConstants.FLAG_CHECKED) && s) {
            // 同时选择AB，并且有S
            requestMap.put(RgaCalcParametersEnum.getParameter(serverSystem, RgaCalcParametersEnum.dataType), RgaDataTypeEnum.IntervalFailureAndSuspension);
        } else if (rgaSetting.getRightFlag().equals(RgaConstants.FLAG_CHECKED) && rgaSetting.getLeftFlag().equals(RgaConstants.FLAG_CHECKED) && !s) {
            // 同时选择AB，并且没有S
            requestMap.put(RgaCalcParametersEnum.getParameter(serverSystem, RgaCalcParametersEnum.dataType), RgaDataTypeEnum.IntervalFailureOnly);
        } else if (rgaSetting.getRightFlag().equals(RgaConstants.FLAG_NO_CHECKED) && rgaSetting.getLeftFlag().equals(RgaConstants.FLAG_NO_CHECKED)) {
            // 都不选择
            requestMap.put(RgaCalcParametersEnum.getParameter(serverSystem, RgaCalcParametersEnum.dataType), RgaDataTypeEnum.ExactFailureOnly);
        } else if (rgaSetting.getRightFlag().equals(RgaConstants.FLAG_CHECKED) && !s) {
            // 选择A且所有的数据均为F时；
            requestMap.put(RgaCalcParametersEnum.getParameter(serverSystem, RgaCalcParametersEnum.dataType), RgaDataTypeEnum.ExactFailureOnly);
        } else if (rgaSetting.getRightFlag().equals(RgaConstants.FLAG_CHECKED) && s) {
            // 选择A且至少有一个数据为S时；
            requestMap.put(RgaCalcParametersEnum.getParameter(serverSystem, RgaCalcParametersEnum.dataType), RgaDataTypeEnum.ExactFailureAndSuspension);
        } else if (rgaSetting.getLeftFlag().equals(RgaConstants.FLAG_CHECKED) && !s) {
            // 选择B且所有的数据均为F时；
            requestMap.put(RgaCalcParametersEnum.getParameter(serverSystem, RgaCalcParametersEnum.dataType), RgaDataTypeEnum.IntervalFailureOnly);
        } else if (rgaSetting.getLeftFlag().equals(RgaConstants.FLAG_CHECKED) && s) {
            // 选择B且至少有一个数据为S时；
            requestMap.put(RgaCalcParametersEnum.getParameter(serverSystem, RgaCalcParametersEnum.dataType), RgaDataTypeEnum.IntervalFailureAndSuspension);
        }
        // 1，选择右删失且所有的数据均为F时;
//		if (altaSetting.getRightFlag().equals("1") && !s) {
//			requestMap.put(AltaCalcParametersEnum.getParameter(serverSystem, AltaCalcParametersEnum.DataType), AltaDataTypeEnum.ExactFailureOnly.getName());
//		}
        // 2，选择左删失且所有的数据均为F时;
//		if (altaSetting.getLeftFlag().equals("1") && !s) {
//			requestMap.put(AltaCalcParametersEnum.getParameter(serverSystem, AltaCalcParametersEnum.DataType), AltaDataTypeEnum.IntervalFailureOnly.getName());
//		}
        // 3，选择右删失且至少有一个数据为S时;
        // 4，选择左删失且至少有一个数据为S时;
        requestMap.put(RgaCalcParametersEnum.getParameter(serverSystem, RgaCalcParametersEnum.dataName), "string");
        requestMap.put(RgaCalcParametersEnum.getParameter(serverSystem, RgaCalcParametersEnum.failureRank), new Integer[0]);
        return requestMap;
    }

    /**
     * 获取快速计算的请求路径
     *
     * @param quickCalcType
     * @param input
     * @return
     */
    public String getQuickCalcUri(Integer quickCalcType, RgaFastCalculateDto input) {
        String uri = null;
        switch (quickCalcType) {
            case 0:
                uri = allVariableConfig.getReliaQubeUrl() + RgaConstants.CHAR_LIFE;
                break;
            case 1:
                uri = allVariableConfig.getReliaQubeUrl() + RgaConstants.RELIABILITY + "?time=" + input.getTime();
                break;
            case 2:
                uri = allVariableConfig.getReliaQubeUrl() + RgaConstants.UN_RELIABILITY + "?time=" + input.getTime();
                break;
            case 3:
                uri = allVariableConfig.getReliaQubeUrl() + RgaConstants.COND_RELIABILITY + "?currentTime=" + input.getCurrentTime() + "&missionTime=" + input.getMissionTime();
                break;
            case 4:
                uri = allVariableConfig.getReliaQubeUrl() + RgaConstants.COND_UN_RELIABILITY + "?currentTime=" + input.getCurrentTime() + "&missionTime=" + input.getMissionTime();
                break;
            case 5:
                uri = allVariableConfig.getReliaQubeUrl() + RgaConstants.FAILURE_RATE + "?time=" + input.getTime();
                break;
            case 6:
                uri = allVariableConfig.getReliaQubeUrl() + RgaConstants.TIME_AT_RELIABILITY + "?Rel=" + input.getRel();
                break;
            case 7:
                uri = allVariableConfig.getReliaQubeUrl() + RgaConstants.TIME_AT_UN_RELIABILITY + "?UnRel=" + input.getUnRel();
                break;
            case 8:
                uri = allVariableConfig.getReliaQubeUrl() + RgaConstants.MEAN_LIFE;
                break;
            case 9:
                uri = allVariableConfig.getReliaQubeUrl() + RgaConstants.ACCELERATION_FACTOR;
                break;
            default:
                uri = BasicDataUtil.EMPTY_STR;
        }
        return uri;
    }

    /**
     * 获取快速计算的上下限请求路径
     *
     * @param quickCalcType
     * @param input
     * @return
     */
    public String getQuickCalcBoundsUri(Integer quickCalcType, RgaFastCalculateDto input) {
        String uri = null;
        switch (quickCalcType) {
            case 0:
                uri = allVariableConfig.getReliaQubeUrl() + RgaConstants.CHAR_LIFE;
                break;
            case 1:
                uri = allVariableConfig.getReliaQubeUrl() + RgaConstants.BOUNDS_RELIABILITY + "?time=" + input.getTime() + "&CL=" + input.getCl() + "&sided=" + input.getSided();
                break;
            case 2:
                uri = allVariableConfig.getReliaQubeUrl() + RgaConstants.BOUNDS_UN_RELIABILITY + "?time=" + input.getTime() + "&CL=" + input.getCl() + "&sided=" + input.getSided();
                break;
            case 3:
                uri = allVariableConfig.getReliaQubeUrl() + RgaConstants.BOUNDS_COND_RELIABILITY + "?currentTime=" + input.getCurrentTime() + "&missionTime=" + input.getMissionTime() + "&CL=" + input.getCl() + "&sided=" + input.getSided();
                break;
            case 4:
                uri = allVariableConfig.getReliaQubeUrl() + RgaConstants.BOUNDS_COND_UN_RELIABILITY + "?currentTime=" + input.getCurrentTime() + "&missionTime=" + input.getMissionTime() + "&CL=" + input.getCl() + "&sided=" + input.getSided();
                break;
            case 5:
                uri = allVariableConfig.getReliaQubeUrl() + RgaConstants.BOUNDS_FAILURE_RATE + "?time=" + input.getTime() + "&CL=" + input.getCl() + "&sided=" + input.getSided();
                break;
            case 6:
                uri = allVariableConfig.getReliaQubeUrl() + RgaConstants.BOUNDS_TIME_AT_RELIABILITY + "?R=" + input.getRel() + "&CL=" + input.getCl() + "&sided=" + input.getSided();
                break;
            case 7:
                uri = allVariableConfig.getReliaQubeUrl() + RgaConstants.BOUNDS_TIME_AT_UN_RELIABILITY + "?UR=" + input.getUnRel() + "&CL=" + input.getCl() + "&sided=" + input.getSided();
                break;
            case 8:
                uri = allVariableConfig.getReliaQubeUrl() + RgaConstants.BOUNDS_MEAN_LIFE + "?CL=" + input.getCl() + "&sided=" + input.getSided();
                break;
            case 9:
                uri = allVariableConfig.getReliaQubeUrl() + RgaConstants.BOUNDS_ACCELERATION_FACTOR + "?CL=" + input.getCl() + "&sided=" + input.getSided();
                break;
            default:
                uri = BasicDataUtil.EMPTY_STR;
                break;
        }
        return uri;
    }
}
